@@ -15,6 +15,7 @@
 #include "parport.h"
 #include "usb.h"
 #include "shm.h"
+#include "servant64.h"
 
 extern Driver* driver;
 
@@ -29,6 +30,9 @@ bool _driver_setup_and_open(void) {
   char default_usb_device[] = "/dev/xlink";
   char default_parport_device[] = "/dev/parport0";
 
+#elif mac
+  char default_usb_device[] = "usb";
+
 #elif windows
   char default_usb_device[] = "usb";
   char default_parport_device[] = "0x378";
@@ -42,12 +46,15 @@ bool _driver_setup_and_open(void) {
 
     if(!(result = driver_setup(default_usb_device))) {
 
+#if linux || windows      
       result = driver_setup(default_parport_device);
 
       if(result) {
         logger->info("using default parallel port device %s",
                      default_parport_device);
       }
+#endif
+      
     }
     else {
       logger->info("using default usb device \"%s\"",
@@ -58,12 +65,14 @@ bool _driver_setup_and_open(void) {
   if(result) {
     result = driver->open();
   }
+#if linux || windows
   else if(autodetect) {
     SET_ERROR(XLINK_ERROR_DEVICE, 
 	      "failed to initialize \"%s\" or \"%s\" (autodetect)",
 	      default_usb_device, default_parport_device);    
   }
-
+#endif
+  
   CLEAR_ERROR_IF(result);
   return result;
 }
@@ -169,8 +178,36 @@ bool driver_setup(char* path) {
     else {
       SET_ERROR(XLINK_ERROR_DEVICE, "failed to initialize shm device \"%s\"", driver->path);
     }
-  }
 
+  } else if(device_is_servant64(type)) {
+    
+    logger->debug("trying to use servant64 device \"%s\"...",  driver->path);
+
+    driver->_open    = &driver_servant64_open;
+    driver->_close   = &driver_servant64_close;    
+    driver->_strobe  = &driver_servant64_strobe;    
+    driver->_wait    = &driver_servant64_wait;
+    driver->_read    = &driver_servant64_read;    
+    driver->_write   = &driver_servant64_write;    
+    driver->_send    = &driver_servant64_send;    
+    driver->_receive = &driver_servant64_receive;    
+    driver->_input   = &driver_servant64_input;    
+    driver->_output  = &driver_servant64_output;    
+    driver->_ping    = &driver_servant64_ping;    
+    driver->_reset   = &driver_servant64_reset;    
+    driver->_boot    = &driver_servant64_boot;    
+    driver->_free    = &driver_servant64_free;
+    
+    result = driver->ready();
+    
+    if(result) {
+      logger->debug("using servant64 device \"%s\"", driver->path);
+    }
+    else {
+      SET_ERROR(XLINK_ERROR_DEVICE, "failed to initialize servant64 device \"%s\"", driver->path);
+    }
+  }  
+  
  done:
   CLEAR_ERROR_IF(result);
   return result;
@@ -205,10 +242,36 @@ bool device_identify(char* path, int* type) {
   CLEAR_ERROR;
   return true;
 
-#elif windows
+#elif mac
+  (*type) = XLINK_DRIVER_DEVICE_USB;
+
+  if(strncmp(path, "usb", 3) == 0) {
+    return true;
+  }
+  
+  struct stat device;
+
+  if(stat(path, &device) == -1) {
+
+    SET_ERROR(XLINK_ERROR_DEVICE, "%s: couldn't stat: %s", path, strerror(errno));
+    return false;
+  }
+
+  if(!S_ISCHR(device.st_mode)) {
+
+    SET_ERROR(XLINK_ERROR_DEVICE, "%s: not a character device", path);
+    return false;
+  }
+
+  (*type) = major(device.st_rdev);
+
+  CLEAR_ERROR;
+  return true;
+
+#elif windows 
 
   (*type) = XLINK_DRIVER_DEVICE_USB;
-  
+
   errno = 0;
   if ((strtol(path, NULL, 0) > 0) && (errno == 0)) {
     (*type) = XLINK_DRIVER_DEVICE_PARPORT;
@@ -223,7 +286,9 @@ bool device_identify(char* path, int* type) {
 bool device_is_supported(char *path, int type) {
 #if linux
 
-  if(!(device_is_parport(type) || device_is_usb(type) || device_is_shm(type))) {
+  if(!(device_is_parport(type) ||
+       device_is_usb(type) ||
+       device_is_shm(type))) {
 
     SET_ERROR(XLINK_ERROR_DEVICE, 
               "%s: unsupported device major number: %d", path, type);
@@ -254,6 +319,12 @@ bool device_is_shm(int type) {
 
 //------------------------------------------------------------------------------
 
+bool device_is_servant64(int type) {
+  return type == XLINK_DRIVER_DEVICE_SERVANT64;
+}
+
+//------------------------------------------------------------------------------
+
 bool _driver_ready() {
   bool result = false;
   
