@@ -1,9 +1,10 @@
 #ifndef DRIVER_H
 #define DRIVER_H
 
-#define XLINK_DRIVER_DEVICE_USB    189
-#define XLINK_DRIVER_DEVICE_PARPORT 99
-#define XLINK_DRIVER_DEVICE_SHM    -1
+#define XLINK_DRIVER_DEVICE_USB       189
+#define XLINK_DRIVER_DEVICE_PARPORT   99
+#define XLINK_DRIVER_DEVICE_SHM       -1
+#define XLINK_DRIVER_DEVICE_SERVANT64 188
 
 #define XLINK_DRIVER_STATE_IDLE   0x01
 #define XLINK_DRIVER_STATE_INPUT  0x02
@@ -48,12 +49,18 @@ typedef struct {
   void (*free) (void);
 } Driver;
 
+typedef struct {
+  uchar *data;
+  uint completed;
+} Transfer;
+
 bool driver_setup(char*);
 bool device_identify(char*, int*); 
 bool device_is_supported(char*, int);
 bool device_is_parport(int);
 bool device_is_usb(int);
 bool device_is_shm(int);
+bool device_is_servant64(int);
 
 bool _driver_setup_and_open(void);
 bool _driver_ready(void);
