@@ -182,7 +182,7 @@ bool driver_usb_open() {
     return false;
   }
 
-  control(USB_INPUT);
+  control(CMD_INPUT);
 
   CLEAR_ERROR;
   return true;
@@ -191,22 +191,22 @@ bool driver_usb_open() {
 //------------------------------------------------------------------------------
 
 void driver_usb_strobe() {
-  control(USB_STROBE);
+  control(CMD_STROBE);
 }
 
 //------------------------------------------------------------------------------
 
+static bool acked(void) {
+  if(controlEndpointIn(CMD_ACKED, response, sizeof(response))) {
+    return response[0] == 1;
+  }
+  return false;
+}
+
 bool driver_usb_wait(int timeout) {
   
   response[0] = 0;
 
-  bool acked() {
-    if(controlEndpointIn(USB_ACKED, response, sizeof(response))) {
-      return response[0] == 1;
-    }
-    return false;
-  }
-
   bool result = false;
 
   if(timeout <= 0) {
@@ -225,69 +225,51 @@ bool driver_usb_wait(int timeout) {
 //------------------------------------------------------------------------------
 
 void driver_usb_write(unsigned char value) {
-  controlEndpointOutWithValue(USB_WRITE, value);
+  controlEndpointOutWithValue(CMD_WRITE, value);
 } 
 
 //------------------------------------------------------------------------------
 
 unsigned char driver_usb_read() {
-  controlEndpointIn(USB_READ, response, sizeof(response));
+  controlEndpointIn(CMD_READ, response, sizeof(response));
   return response[0];
 } 
 
 //------------------------------------------------------------------------------
 
-static bool chunked(bool (*callback) (ushort chunk), ushort chunk, int size) {
+static bool send_chunk(ushort chunk, void *context) {
 
-  bool result = true;
-
-  do {
-    if(size > chunk) {
-      size -= chunk;
-    }
-    else {
-      chunk = size;
-      size = 0;
-    }
-    
-    if(!(result = (*callback)(chunk))) {
-      break;
-    }    
-  } while(size > 0);
-  return result;
+  Transfer *transfer = (Transfer*) context;
+  
+  int transfered = controlEndpointOut(CMD_SEND, transfer->data, chunk);
+  
+  if(transfered < 0) {
+    return false;
+  }
+  
+  if (transfered < chunk) { 
+    transfer->completed += transfered;
+    return false;
+  }
+  
+  transfer->data += chunk;
+  transfer->completed += chunk;
+  return true;
 }
 
-//------------------------------------------------------------------------------
-
 bool driver_usb_send(unsigned char* data, int size) {
 
-  int sent = 0;
+  Transfer transfer;
+  transfer.data = data;
+  transfer.completed = 0;
   
-  bool send(ushort chunk) {
+  chunked(send_chunk, &transfer, MAX_PAYLOAD_SIZE, size);
 
-    int transfered = controlEndpointOut(USB_SEND, data, chunk);
-
-    if(transfered < 0) {
-      return false;
-    }
-
-    if (transfered < chunk) { 
-      sent += transfered;
-      return false;
-    }
-    
-    data += chunk;
-    sent += chunk;
-    return true;
-  }
-
-  chunked(send, MAX_PAYLOAD_SIZE, size);
-
-  bool result = sent == size;
+  bool result = transfer.completed == size;
   
   if(!result) {
     SET_ERROR(XLINK_ERROR_LIBUSB,
-              "transfer timeout (%d of %d bytes sent)", sent, size);
+              "transfer timeout (%d of %d bytes sent)", transfer.completed, size);
   }
   
   CLEAR_ERROR_IF(result);
@@ -296,34 +278,39 @@ bool driver_usb_send(unsigned char* data, int size) {
 
 //------------------------------------------------------------------------------
 
-bool driver_usb_receive(unsigned char* data, int size) {
-  int received = 0;
+static bool receive_chunk(ushort chunk, void* context) {
+
+  Transfer *transfer = (Transfer*) context;
   
-  bool receive(ushort chunk) {
-
-    int transfered = controlEndpointIn(USB_RECEIVE, data, chunk);
-
-    if(transfered < 0) { 
-      return false;
-    }
-
-    if (transfered < chunk) { 
-      received += transfered;
-      return false;
-    }
-    
-    data += chunk;
-    received += chunk;
-    return true;
+  int transfered = controlEndpointIn(CMD_RECEIVE, transfer->data, chunk);
+  
+  if(transfered < 0) { 
+    return false;
   }
+  
+  if (transfered < chunk) { 
+    transfer->completed += transfered;
+    return false;
+  }
+  
+  transfer->data += chunk;
+  transfer->completed += chunk;
+  return true;
+}
 
-  chunked(receive, MAX_PAYLOAD_SIZE, size);
+bool driver_usb_receive(unsigned char* data, int size) {
 
-  bool result = received == size;
+  Transfer transfer;
+  transfer.data = data;
+  transfer.completed = 0;
+  
+  chunked(&receive_chunk, &transfer, MAX_PAYLOAD_SIZE, size);
+
+  bool result = transfer.completed == size;
   
   if(!result) {
     SET_ERROR(XLINK_ERROR_LIBUSB,
-              "transfer timeout (%d of %d bytes received)", received, size);
+              "transfer timeout (%d of %d bytes received)", transfer.completed, size);
   }
   
   CLEAR_ERROR_IF(result);
@@ -333,13 +320,13 @@ bool driver_usb_receive(unsigned char* data, int size) {
 //------------------------------------------------------------------------------
 
 void driver_usb_input() {
-  control(USB_INPUT);
+  control(CMD_INPUT);
 }
 
 //------------------------------------------------------------------------------
 
 void driver_usb_output() {
-  control(USB_OUTPUT);
+  control(CMD_OUTPUT);
 }
 
 //------------------------------------------------------------------------------
@@ -354,13 +341,13 @@ bool driver_usb_ping() {
 //------------------------------------------------------------------------------
 
 void driver_usb_reset() { 
-  control(USB_RESET);
+  control(CMD_RESET);
 }
 
 //------------------------------------------------------------------------------
 
 void driver_usb_boot() { 
-  control(USB_BOOT);
+  control(CMD_BOOT);
 }
 
 //------------------------------------------------------------------------------
