@@ -5,11 +5,10 @@
 #include <time.h>
 
 #include "xlink.h"
+#include "machine.h"
 #include "error.h"
 #include "target.h"
 #include "driver/driver.h"
-#include "extension.h"
-#include "extensions.c"
 #include "util.h"
 
 #if windows
@@ -17,6 +16,7 @@
 #endif
 
 #define XLINK_DEFAULT_TIMEOUT  0x01
+#define XLINK_GO64 0xff4d
 
 Driver* driver;
 xlink_error_t* xlink_error;
@@ -35,6 +35,7 @@ void xlink_set_debug(bool enabled) {
 
 //------------------------------------------------------------------------------
 
+__attribute((constructor))
 void libxlink_initialize() {
 
   driver = (Driver*) calloc(1, sizeof(Driver));
@@ -60,6 +61,16 @@ void libxlink_initialize() {
 
   driver->_open = &_driver_setup_and_open;
 
+  if (getenv("XLINK_MACHINE") != NULL) {
+
+    if (strncmp(getenv("XLINK_MACHINE"), "c64", 3) == 0) {
+      machine = &c64;
+    }
+    if (strncmp(getenv("XLINK_MACHINE"), "c128", 4) == 0) {
+      machine = &c128;
+    }
+  }
+  
   xlink_error = (xlink_error_t *) calloc(1, sizeof(xlink_error_t));
   CLEAR_ERROR;
 
@@ -68,6 +79,7 @@ void libxlink_initialize() {
 
 //------------------------------------------------------------------------------
 
+__attribute((destructor))
 void libxlink_finalize(void) {
   
   if(driver != NULL) {
@@ -207,24 +219,48 @@ bool xlink_ping() {
 //------------------------------------------------------------------------------
 
 bool xlink_reset(void) {
+
   bool result = false;
   
   if(driver->open()) {
     driver->reset();
     driver->close();
-    result = true;
+
+    if(machine->type == XLINK_MACHINE_C128) {
+      while(xlink_ping()); 
+    }
+    
+    return true;
   }
-  
+ 
   CLEAR_ERROR_IF(result);
   return result;
 };
 
 //------------------------------------------------------------------------------
 
+static bool server_ready_after(int ms) {
+
+  logger->trace("waiting for server...");
+  
+  while(ms) {
+    if(xlink_ping()) {
+      usleep(250*1000);
+      return true;
+    }
+    ms-=250;
+  }
+  return false;
+}
+
+//------------------------------------------------------------------------------
+
 bool xlink_ready(void) {
 
   bool result = true;
   int timeout = 3000;
+  
+  xlink_server_info_t remote;
   unsigned char mode;
 
   if(!driver->ready()) {
@@ -232,28 +268,52 @@ bool xlink_ready(void) {
     goto done;
   }
 
-  if(!xlink_ping()) {
+  if(!xlink_ping() || !xlink_ping()) {
+    logger->trace("server not reachable, resetting...");
+
     xlink_reset();
+    usleep(100*1000);
     
-    while(timeout) {
-      if(xlink_ping()) {
-        usleep(250*1000); // wait until basic is ready
-        goto done;
-      }
-      timeout-=250;
-    }
-    result = false;
+    if(!(result = server_ready_after(timeout))) {
+      goto done;
+    }    
   }
-  else {
-    if(xlink_peek(0x37, 0x00, 0x9d, &mode)) {
-      if(mode != 0x80) { 
-        // basic program running -> warm start basic
-        xlink_jump(0x37, 0x00, 0xfe66);
-        usleep(250*1000);
+
+  if(machine->type == XLINK_MACHINE_C64) {
+    if((result = xlink_identify(&remote))) {
+      if(remote.machine == XLINK_MACHINE_C128) {
+        logger->trace("C128 server identified, switching to C64 mode");
+        if((result = xlink_jump(c128.memory, c128.bank, XLINK_GO64))) {
+          result = server_ready_after(timeout);
+          goto done;	  
+        }
+      }
+    }
+  }
+
+  if(machine->type == XLINK_MACHINE_C128) {
+    if((result = xlink_identify(&remote))) {
+      if(remote.machine == XLINK_MACHINE_C64) {
+        logger->trace("C64 server identified, switching to C128 mode");
+        if((result = xlink_reset())) {
+          result = server_ready_after(timeout);
+          goto done;	  
+        }
       }
     }
   }
   
+  if(result) {
+    if((result = xlink_peek(machine->memory, machine->bank, machine->mode, &mode))) {
+      if(mode == machine->prgmode) {
+        logger->debug("basic program running, performing basic warmstart...");
+        if((result = xlink_jump(machine->memory, machine->bank, machine->warmstart))) {
+          usleep(250*1000);
+        }
+      }
+    }
+  }
+
  done:
   CLEAR_ERROR_IF(result);
   return result;
@@ -511,8 +571,8 @@ bool xlink_run(void) {
 bool xlink_inject(ushort address, uchar* code, uint size) {
 
   bool result = false;
-  uchar memory = 0x80|0x37;
-  uchar bank = 0x00;
+  uchar memory = machine->memory;
+  uchar bank = machine->bank;
   
   if(!xlink_load(memory, bank, address, code, size)) {
     goto done;
@@ -643,183 +703,25 @@ void xlink_end() {
 
 //------------------------------------------------------------------------------
 
-extern unsigned char* xlink_server(unsigned short address, int *size);
-
 bool xlink_relocate(unsigned short address) {
 
   bool result = false;
   
-  Extension* relocate = EXTENSION_SERVER_RELOCATE;
-
   int size;
-  unsigned char* server = xlink_server(address, &size);  
+  unsigned char* server = machine->server(address, &size);  
 
-  if(extension_init(relocate)) {
-    result = xlink_load(0x37|0x80, 0x00, address, server+2, size-2);
-  }
+  uchar memory = machine->memory | 0x80;
+  uchar bank = machine->bank;
+  
+  if(!(result = xlink_load(memory, bank, address, server+2, size-2))) goto done;
+
+  result = xlink_jump(memory, bank, address);
   
-  extension_free(relocate);  
   free(server);
-  
+
+ done:
   CLEAR_ERROR_IF(result);
   return result;  
 }
 
 //------------------------------------------------------------------------------
-
-bool xlink_drive_status(char* status) {
-
-  unsigned char byte;
-  bool result = false;
-
-  Extension *lib = EXTENSION_LIB;
-  Extension *drive_status = EXTENSION_DRIVE_STATUS;
-
-  if (extension_preload(lib) && 
-      extension_init(drive_status)) {
-
-    if (driver->open()) {
-      
-      driver->input();
-      driver->strobe();
-
-      int i = 0;
-
-      while(true) {
-
-        driver->receive(&byte, 1);
-
-        if(byte == 0xff) break;
-
-        status[i++] = byte;	
-      }
-      driver->wait(0);
-
-      driver->close();
-      result = true;
-    }
-    extension_unload(lib);
-  }
-
-  extension_free(lib);
-  extension_free(drive_status);
-  
-  CLEAR_ERROR_IF(result);
-  return result;
-}
-
-//------------------------------------------------------------------------------
-
-bool xlink_dos(char* cmd) {
-
-  bool result = false;
-
-  Extension *lib = EXTENSION_LIB;
-  Extension *dos_command = EXTENSION_DOS_COMMAND;
-
-  char *command = (char *) calloc(strlen(cmd)+1, sizeof(char));
-  
-  for(int i=0; i<strlen(cmd); i++) {
-    command[i] = toupper(cmd[i]);
-  }      
-
-  if(extension_preload(lib) && 
-     extension_init(dos_command)) {
-    
-    if(driver->open()) {   
-
-      driver->output();
-      driver->send((unsigned char []) {strlen(command)}, 1);
-      driver->send((unsigned char*) command, strlen(command));
-
-      driver->wait(0);
-
-      driver->close();
-      result = true;
-    }
-    extension_unload(lib);
-  }
-  
-  free(command);
-  extension_free(lib);
-  extension_free(dos_command);
-
-  CLEAR_ERROR_IF(result);
-  return result;
-}
-
-//------------------------------------------------------------------------------
-
-bool xlink_sector_read(unsigned char track, unsigned char sector, unsigned char* data) {
-  
-  bool result = false;
-  char U1[13];
-
-  Extension *lib = EXTENSION_LIB;
-  Extension *sector_read = EXTENSION_SECTOR_READ;
-
-  if (extension_preload(lib) && 
-      extension_init(sector_read)) {
-
-    if (driver->open()) {
-      
-      sprintf(U1, "U1 2 0 %02d %02d", track, sector);
-
-      driver->output();
-      driver->send((unsigned char*)U1, strlen(U1));
-
-      driver->input();
-      driver->strobe();
-
-      driver->receive(data, 256);
-      driver->wait(0);
-      
-      driver->close();      
-      result = true;
-    }
-    extension_unload(lib);
-  }
-
-  extension_free(lib);
-  extension_free(sector_read);
-
-  CLEAR_ERROR_IF(result);
-  return result;
-}
-
-//------------------------------------------------------------------------------
-
-bool xlink_sector_write(unsigned char track, unsigned char sector, unsigned char *data) {
-
-  int result = false;
-  char U2[13];
-
-  Extension *lib = EXTENSION_LIB;
-  Extension *sector_write = EXTENSION_SECTOR_WRITE;
-
-  if (extension_preload(lib) && 
-      extension_init(sector_write)) {
-
-    if (driver->open()) {
-      sprintf(U2, "U2 2 0 %02d %02d", track, sector);
-
-      driver->output();
-      driver->send(data, 256);
-      driver->send((unsigned char*)U2, strlen(U2));
-
-      driver->wait(0);
-      
-      driver->close();
-      result = true;
-    }
-    extension_unload(lib);
-  }
-  
-  extension_free(lib);
-  extension_free(sector_write);
-
-  CLEAR_ERROR_IF(result);
-  return result;
-}
-
-//------------------------------------------------------------------------------
