@@ -17,6 +17,7 @@
 #define XLINK_SERVER_TYPE_RAM  0x00
 #define XLINK_SERVER_TYPE_ROM  0x01
 #define XLINK_MACHINE_C64      0x00
+#define XLINK_MACHINE_C128     0x01
 
 #define XLINK_SUCCESS          0x00
 #define XLINK_ERROR_DEVICE     0x01
@@ -24,6 +25,7 @@
 #define XLINK_ERROR_PARPORT    0x03
 #define XLINK_ERROR_SERVER     0x04
 #define XLINK_ERROR_FILE       0x05
+#define XLINK_ERROR_SERVANT64  0x06
 
 #define XLINK_COMMAND_LOAD     0x01
 #define XLINK_COMMAND_SAVE     0x02
